@@ -56,9 +56,9 @@ if [ "$ACTION" == "-i" ]; then
 		/usr/sbin/semodule -i snare.pp
 	fi
 
-	echo "# SnareDispatcher added" > $CONFDIR/$AUDITCONF
+	echo "# SnareDispatchHelper added" > $CONFDIR/$AUDITCONF
 	cat $CONFDIR/$AUDITCONF-snarebackup | sed 's/^dispatcher.*//' | sed 's/^log_format =.*/log_format = NOLOG/' | egrep -v "^$" >> $CONFDIR/$AUDITCONF
-	echo "dispatcher = $BINDIR/SnareDispatcher" >> $CONFDIR/$AUDITCONF
+	echo "dispatcher = $BINDIR/SnareDispatchHelper" >> $CONFDIR/$AUDITCONF
 	cat $CONFDIR/$AUDITRULES-snarebackup | sed 's/^-D.*//' | egrep -v "^$" > $CONFDIR/$AUDITRULES
 
 	$CONFDIR/init.d/auditd start
@@ -67,23 +67,26 @@ fi
 
 if [ $1 == "-u" ]; then
 	# Uninstall
-	$CONFDIR/init.d/auditd stop
-	if [ -f $CONFDIR/$AUDITCONF-snarebackup ]; then
-                cat $CONFDIR/$AUDITCONF-snarebackup | sed 's/^log_format =.*/log_format = RAW/' | egrep -v "^dispatcher = .*SnareDispatcher.*" > $CONFDIR/$AUDITCONF
-		rm -f $CONFDIR/$AUDITCONF-snarebackup
-        else
-                cp $CONFDIR/$AUDITCONF $CONFDIR/$AUDITCONF-snarebackup
-                cat $CONFDIR/$AUDITCONF-snarebackup | sed 's/^log_format =.*/log_format = RAW/' | egrep -v "^dispatcher" > $CONFDIR/$AUDITCONF
-                rm -f $CONFDIR/$AUDITCONF-snarebackup
-        fi
-
-	if [ -f /usr/sbin/semodule ]; then
-		/usr/sbin/semodule -l | egrep "^snare" > /dev/null
-		if [ "$?" -eq 0 ]; then
-			/usr/sbin/semodule -r snare;
+	UNINST=`rpm -q SnareLinux | wc -l`
+	if [ $UNINST -le 1 ]; then
+		$CONFDIR/init.d/auditd stop
+		if [ -f $CONFDIR/$AUDITCONF-snarebackup ]; then
+			cat $CONFDIR/$AUDITCONF-snarebackup | sed 's/^log_format =.*/log_format = RAW/' | egrep -v "^dispatcher = .*SnareDispatch.*" > $CONFDIR/$AUDITCONF
+			rm -f $CONFDIR/$AUDITCONF-snarebackup
+		else
+			cp $CONFDIR/$AUDITCONF $CONFDIR/$AUDITCONF-snarebackup
+			cat $CONFDIR/$AUDITCONF-snarebackup | sed 's/^log_format =.*/log_format = RAW/' | egrep -v "^dispatcher" > $CONFDIR/$AUDITCONF
+			rm -f $CONFDIR/$AUDITCONF-snarebackup
 		fi
-	fi
 
-	$CONFDIR/init.d/auditd start
+		if [ -f /usr/sbin/semodule ]; then
+			/usr/sbin/semodule -l | egrep "^snare" > /dev/null
+			if [ "$?" -eq 0 ]; then
+				/usr/sbin/semodule -r snare;
+			fi
+		fi
+
+		$CONFDIR/init.d/auditd start
+	fi
 	exit
 fi
