@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
 # Snare Audit Dispatcher for Linux
-# (c) Copyright 2006 InterSect Alliance Pty Ltd
+# (c) Copyright 2007 InterSect Alliance Pty Ltd
 #
 # This application will integrate into the native linux audit subsystem,
 # and translate linux auditd log format data into something that can be
@@ -13,18 +13,10 @@
 #	* Run the MakeTranslationTable program
 #	  (optional, but highly recommended)
 #         It will create /etc/snare-xlate.conf
+#	* Copy SnareDispatchHelper to /usr/sbin/SnareDispatchHelper
 #	* Save this file to /usr/sbin/SnareDispatcher.pl
 #	* Add the following line to your /etc/auditd.conf file:
-#		dispatcher = /usr/sbin/SnareDispatcher.pl
-#
-# Notes:
-# Although the items=x field should provide us with a reasonable guestimate
-# of how many lines to expect for a particular syscall event, items=x isn't
-# available for all events. As such, as well as a items=x test, the code uses
-# a timeout to determine when to send an event, as we can't easily establish
-# exactly how many event-lines might potentially contribute to a single event
-# without items=x (even then, we are assuming that the PATH line will be the
-# last to be sent). Yuck.
+#		dispatcher = /usr/sbin/SnareDispatchHelper
 #
 # Issues:
 #	* a0, a1, a2, a3 are practically useless, as string arguments are not
@@ -69,9 +61,6 @@ $MANAGEAUDIT=1;
 # Should we use regular expressions instead of wildcards?
 $REGEX=0;
 
-# How often should we flush our buffer?
-$FLUSHINTERVAL=10;	# in seconds
-
 $WEBSERVERALLOW=0;
 $WEBSERVERPORT=6161;
 $ACCESSKEY="";
@@ -82,10 +71,8 @@ $SYSLOGDEST=0;
 
 ############# End user configurable options.
 
-$VERSION="1.2";
+$VERSION="1.3";
 
-$FLUSH=0;
-$FLUSHNOW=-1;
 $CONTINUE=1;
 
 # Open our translation table.
@@ -97,6 +84,10 @@ if($rc) {
 		$syscall{$num}=$scall;
 		$nsyscall{$call}=$num;
 	}
+	$syscall{-1}="login_auth";
+	$syscall{-2}="login_start";
+	$syscall{-3}="logout";
+	$syscall{-4}="acct_mgmt";
 	close(XLATE);
 }
 
@@ -104,9 +95,10 @@ if($rc) {
 # Declare a UID/GID cache so we don't wind up getting into recursive
 # audit event generation.
 my %uidcache=();
+my %unamecache=();
 my %gidcache=();
 
-# Global event cache
+# Event cache
 %event=();
 
 # Global filters array
@@ -114,7 +106,7 @@ my %gidcache=();
 $eventsent=0;
 $lasteventnum=0;
 
-$timecheck=time();
+$DNSTIMEOUT=600;
 
 $FQDN=`hostname --fqdn`;
 chomp($FQDN);
@@ -141,31 +133,24 @@ $SIG{QUIT}='IGNORE';
 $SIG{TERM}='Handler';
 
 LogMsg("Snare for Linux $VERSION: Started and active.");
+$DNSNOW=time();
+$DNSLAST=$DNSNOW;
 
 # Main reader loop.
 while($CONTINUE==1) {
-	$now=time();
-	# Every 10 seconds
-	if($now-$timecheck > $FLUSHINTERVAL) {
-		# Flush any old data
-		SendEvents($FLUSH);
-		$timecheck=$now;
-	}
-
 	$nfound=select($rout=$rin,undef,undef,1);
 	if($nfound > 0) {
 		# Grab the version number
-		$bytes=sysread(STDIN,$tversion,4);
+		$bytes=sysread(STDIN,$header,8);
 		if($bytes==0) {
+			if ($DEBUG) { LogMsg("DEBUG: Failed to get header\n"); }
 			$CONTINUE=0;
 			last;
 		} elsif ($bytes==undef) {
 			# Probably interrupted by a signal
 			next;
 		}
-		sysread(STDIN,$theadersize,4);
-		($version)=unpack("I",$tversion);
-		($headersize)=unpack("I",$theadersize);
+		($version,$headersize)=unpack("II",$header);
 
 		# Eventtype 1302 = PATH
 		# Eventtype 1300 = SYSCALL
@@ -185,7 +170,7 @@ while($CONTINUE==1) {
 			next;
 		}
 	} else {
-		if($DEBUG>1) { LogMsg("NO DATA (timout)!"); }
+		if($DEBUG>1) { LogMsg("NO DATA (timeout)!"); }
 		next;
 	}
 
@@ -211,8 +196,13 @@ while($CONTINUE==1) {
 
 
 	# Lets split this line up into element/content pairs.
-	# First, break apart by spaces that aren't inside inverted commas.
-	@elements=split(/\s(?=(?:[^"]*"[^"]*")*[^"]*\z)/,$line);  # comment helper """
+	# First, lets check for msg strings and strip them out (to deal with them later)
+	if ($line =~ /(.*) msg='([^']*)'(.*)/) {		# Make gedit pretty: '
+		$msg = $2;
+		$line = $1.$3;
+	}
+	# Next, break apart by spaces that aren't inside inverted commas.
+	@elements=split(/\s+(?=(?:[^"]*"[^"]*")*[^"]*\z)/,$line);  # comment helper """
 	%data=();
 	if($DEBUG>2) { LogMsg("Elements contains " . @elements . " elements"); }
 	foreach $element (@elements) {
@@ -224,6 +214,29 @@ while($CONTINUE==1) {
 			$data{$key}=$val;
 		}
 	}
+	# Now, do the same thing for the msg string, but add a few special clauses
+	# Strip out any brackets
+	undef $key;
+	$msg =~ s/[\(\)]//g;
+	@elements=split(/[,]?\s(?=(?:[^"]*"[^"]*")*[^"]*\z)/,$msg);  # comment helper """
+	if($DEBUG>2) { LogMsg("Elements contains " . @elements . " elements"); }
+	foreach $element (@elements) {
+		if($element=~/=/) {
+			# Then, by the equals sign.
+			($key,$val)=split(/=/,$element,2);
+			$val =~ s/^"//;		# Make gedit pretty: "
+			$val =~ s/"$//;		# Make gedit pretty: "
+			$data{$key}=$val;
+		} else {
+			if (!defined($key)) {
+				$key = "msg";
+				$data{$key} = $element;
+				next;
+			}
+			# add the term to the previous key
+			$data{$key} .= " $element";
+		}
+	}
 
 	# Ok, we now have an array of key/value pairs.
 
@@ -234,8 +247,6 @@ while($CONTINUE==1) {
 		$sdatetime=strftime("%Y%m%d %T",@ltime);
 		$event{$eventnum}{"datetime"}=$sdatetime;
 	}
-	# Unix time..
-	$event{$eventnum}{"utime"}=$datetime;
 
 	foreach $key (keys %data) {
 		$count=0;
@@ -247,24 +258,65 @@ while($CONTINUE==1) {
 	}
 
 	# Eventtype 1302 = "PATH"
+	# Eventtype 1300 = SYSCALL
+	# Eventtype 1112 = "USER_LOGIN"
 	# This should tell us when the end of a record comes through.
-	if($eventtype == 1302) {
-		$items=$event{$eventnum}{"items"};
-		if($items > 0) {
-			$pathcount=$event{$eventnum}{"pathcount"};
-			$pathcount++;
-			if($pathcount == $items) {
-				# We have all the data we need! Send this event out the door.
-				# Clear our counter first.
-				delete $event{$eventnum}{"pathcount"};
-				if($DEBUG>1) { LogMsg("Sending out Event $eventnum - we've had $pathcount items ($items)!"); }
-				SendEvents($eventnum);
+	if($eventtype == 1300) {
+		# We have all the data we need! Send this event out the door.
+		if($DEBUG>1) { LogMsg("Sending out Event $eventnum"); }
+		SendEvents($eventnum);
+	} elsif($eventtype == 1112 || $eventtype == 1100 || $eventtype == 1105 || $eventtype == 1106 || $eventtype == 1108) {
+#1112,1100,1105 - authentication
+#1106 - USER_END
+#1108 - USER_CHAUTHTOK
+#1104 - ??
+		if($DEBUG>1) { LogMsg("User Login event detected, formatting and sending"); }
+		if($DEBUG>1) { LogMsg("Sending out Event $eventnum"); }
+		# set up some of the missing variables
+		$event{$eventnum}{"gid"}=0;
+		$event{$eventnum}{"exit"}=0;
+		if (defined($event{$eventnum}{"terminal"})) {
+			$event{$eventnum}{"comm"}=$event{$eventnum}{"terminal"};
+		} else {
+			$event{$eventnum}{"comm"}="-";
+		}
+		$event{$eventnum}{"success"} = "no";
+		if (defined($event{$eventnum}{"res"}) && $event{$eventnum}{"res"} =~ /success/i) {
+			$event{$eventnum}{"success"} = "yes";
+			delete $event{$eventnum}{"res"};
+		} elsif (defined($event{$eventnum}{"result"}) && $event{$eventnum}{"result"} =~ /Success/i) {
+			$event{$eventnum}{"success"} = "yes";
+			delete $event{$eventnum}{"result"};
+		} elsif (!defined($event{$eventnum}{"res"}) && !defined($event{$eventnum}{"result"}) && $event{$eventnum}{"msg"} !~ /fail/i) {
+			$event{$eventnum}{"success"} = "yes";
+		} else {
+			$event{$eventnum}{"success"} = "no";
+		}
+		# login/logout specifics
+		if ($eventtype == 1106) {
+			#USER_END
+			$event{$eventnum}{"syscall"}=-3;
+		} elsif ($eventtype == 1108) {
+			# USER_CHAUTHTOK
+			$event{$eventnum}{"syscall"}=-4;
+		} else {
+			#USER_AUTH, USER_START
+			if ($eventtype == 1105) {
+				$event{$eventnum}{"syscall"}=-2;
 			} else {
-				if($DEBUG>1) { LogMsg("Found another PATH component for event $eventnum."); }
-				$event{$eventnum}{"pathcount"}++;
+				$event{$eventnum}{"syscall"}=-1;
 			}
 		}
+		if (!defined($event{$eventnum}{"msg"})) {
+			$event{$eventnum}{"msg"} = $event{$eventnum}{"op"};
+		} else {
+			$event{$eventnum}{"msg"} .= " " . $event{$eventnum}{"op"};
+		}
+		delete $event{$eventnum}{"op"};
+		delete $event{$eventnum}{"res"};
+		SendEvents($eventnum);
 	}
+	delete $event{$eventnum};
 }
 
 if ($DEBUG) { LogMsg("DEBUG: Exit requested\n"); }
@@ -276,241 +328,227 @@ if($webpid) {
 }
 
 # Flush our buffers
-SendEvents($FLUSHNOW);
 CloseOutputs();
 
 exit;
 
 sub SendEvents {
-	($eventnumber)=@_;
+	($eventnum)=@_;
 
-	@sendevents=();
-
-	$now=time();
 	# If we are sent a particular event number, flush it out.
-	if($eventnumber != $FLUSH && $eventnumber != $FLUSHNOW) {
-		@sendevents=($eventnumber);
-	} else {
-		# if eventnumber is zero, flush any events that are older than 10 seconds.
-		# otherwise, if eventnumber is <0, flush all events we have - We're going down.
-		foreach $eventnum (sort keys %event) {
-			$utime=$event{$eventnum}{"utime"};
-			if(($now-$utime < $FLUSHINTERVAL) && $eventnumber==$FLUSH) {
-				# Since we're sorted, we can break out as soon as we find a event in the last 5 secs.
-				last;
-			}
-			delete $event{$eventnum}{"utime"};
-			push(@sendevents,$eventnum);
-		}
-	}
-
-	# Ok, we now have an array of events to send out.
-	foreach $eventnum (@sendevents) {
 		#if($DEBUG) { LogMsg("DEBUG: Eventnum is $eventnum"); }
 
-		# If the event doesn't satisfy the basic requirements, dump it
-		if(!defined($event{$eventnum}{"syscall"}) ||
-		   !defined($event{$eventnum}{"datetime"}) ||
-		   !defined($event{$eventnum}{"uid"}) ||
-		   !defined($event{$eventnum}{"gid"}) ||
-		   !defined($event{$eventnum}{"pid"}) ||
-		   !defined($event{$eventnum}{"comm"}) ||
-		   !defined($event{$eventnum}{"exit"}) ||
-		   !defined($event{$eventnum}{"success"})) {
-			if($DEBUG) {
-				LogMsg(":::::::: Event $eventnum does not contain all required data");
-				@elements=keys %{$event{$eventnum}};
-				@values=values %{$event{$eventnum}};
-				LogMsg(":::::::: [@elements] [@values]");
-			}
-
-			delete $event{$eventnum};
-
-			next;
-		}
-	
-
-		# Lets construct a token/data array in the order we wish to send things out, with
-		# appropriate data extrapolated.
-		my %tokens=();
-		my @tokenlist=();
-
-		$syscallname=$syscall{$event{$eventnum}{"syscall"}};
-		if(!$syscallname) {
-			# Fall back to the number.
-			$syscallname=$event{$eventnum}{"syscall"};
-		}
-		delete $event{$eventnum}{"syscall"};
-		$tokens{"event"}="$syscallname," . $event{$eventnum}{"datetime"};
-		push(@tokenlist,"event");
-		delete $event{$eventnum}{"datetime"};
-		$uidname=getuid($event{$eventnum}{"uid"});
-		$tokens{"uid"}=$event{$eventnum}{"uid"} . ($uidname?",$uidname":"");
-		push(@tokenlist,"uid");
-		delete $event{$eventnum}{"uid"};
-		$gidname=getgid($event{$eventnum}{"gid"});
-		$tokens{"gid"}=$event{$eventnum}{"gid"} . ($uidname?",$gidname":"");
-		push(@tokenlist,"gid");
-		delete $event{$eventnum}{"gid"};
-		$euidname=getuid($event{$eventnum}{"euid"});
-		$tokens{"euid"}=$event{$eventnum}{"euid"} . ($uidname?",$euidname":"");
-		push(@tokenlist,"euid");
-		delete $event{$eventnum}{"euid"};
-		$egidname=getgid($event{$eventnum}{"egid"});
-		$tokens{"egid"}=$event{$eventnum}{"egid"} . ($uidname?",$egidname":"");
-		push(@tokenlist,"egid");
-		delete $event{$eventnum}{"egid"};
-		$tokens{"process"}=$event{$eventnum}{"pid"} . "," . $event{$eventnum}{"comm"};
-		push(@tokenlist,"process");
-		delete $event{$eventnum}{"pid"};
-		delete $event{$eventnum}{"comm"};
-		$tokens{"return"}=$event{$eventnum}{"exit"} . "," . $event{$eventnum}{"success"};
-		push(@tokenlist,"return");
-		delete $event{$eventnum}{"exit"};
-		delete $event{$eventnum}{"success"};
-
-
-		foreach $key (sort keys %{$event{$eventnum}} ) {
-			# Resolve names
-			if($key =~ /^[eosa]uid(:[0-9]+)*$/ || $key =~ /^fsuid(:[0-9]+)*$/) {
-				$name=getuid($event{$eventnum}{$key});
-				if($name) {
-					$eventstring = $event{$eventnum}{$key} . ",$name";
-				}
-			} elsif($key =~ /^[eosa]gid(:[0-9]+)*$/ || $key =~ /^fsgid(:[0-9]+)*$/) {
-				$name=getgid($event{$eventnum}{$key});
-				if($name) {
-					$eventstring = $event{$eventnum}{$key} . ",$name";
-				}
-			} elsif($key =~ /^name(:[0-9]+)*$/ || $key =~ /^exe(:[0-9]+)*$/) {
-				$tpath=$event{$eventnum}{$key};
-				if($tpath =~ /^\// || !$event{$eventnum}{"cwd"}) {
-					$path = resolve_path($tpath);
-				} else {
-					$path = resolve_path($event{$eventnum}{"cwd"} . "/" . $tpath);
-				}
-				$eventstring = $path;
-			} else {
-				$eventstring = $event{$eventnum}{$key};
-			}
-			$tokens{$key}=$eventstring;
-			push(@tokenlist,$key);
+	# If the event doesn't satisfy the basic requirements, dump it
+	if(!defined($event{$eventnum}{"syscall"}) ||
+	   !defined($event{$eventnum}{"datetime"}) ||
+	   !defined($event{$eventnum}{"uid"}) ||
+	   !defined($event{$eventnum}{"gid"}) ||
+	   !defined($event{$eventnum}{"pid"}) ||
+	   !defined($event{$eventnum}{"comm"}) ||
+	   !defined($event{$eventnum}{"exit"}) ||
+	   !defined($event{$eventnum}{"success"})) {
+		if($DEBUG) {
+			LogMsg(":::::::: Event $eventnum does not contain all required data");
+			@elements=keys %{$event{$eventnum}};
+			@values=values %{$event{$eventnum}};
+			LogMsg(":::::::: [@elements] [@values]");
 		}
 
-		# Ok, we have a range of token/value pairs to use for event checks.
 		delete $event{$eventnum};
 
-		# Format of the filters:
-		#$Filters{$ObjectiveCount}{$key}{$keymatch}{$ElementCount}{$AlternativeCount}=$alternative;
-		#$Filters{0}{event}{0}{0}{0}="execve";
-		$EventMatched=0;
-		$EventCrit=0;
-		foreach $objectivecount (keys(%Filters)) {
-			$ObjectiveMatch=0;
-			foreach $key (keys(%{$Filters{$objectivecount}})) {
-				if($tokens{$key}) {
-					foreach $keymatch (keys(%{$Filters{$objectivecount}{$key}})) {
-						foreach $elementcount (keys(%{$Filters{$objectivecount}{$key}{$keymatch}})) {
-							@elements=split(/,/,$tokens{$key});
-							if($DEBUG>3) { LogMsg("Key: $key - ELEMENTS: @elements\n"); }
-							$result=0;
-							foreach $alternativecount (keys(%{$Filters{$objectivecount}{$key}{$keymatch}{$elementcount}})) {
-								$match=$Filters{$objectivecount}{$key}{$keymatch}{$elementcount}{$alternativecount};
-								$element=$elements[$elementcount];
-								$negate=$FilterTypes{$objectivecount}{$key}{$keymatch};
-								if($DEBUG>2) { LogMsg("Match element $element against $match (negate is $negate)\n"); }
+		next;
+	}
 
-								# Special case:
-								if($match eq "*") {
-									$result=1;
-								} else {
-									@strings=split(/,/,$element);
-									@matches=split(/,/,$match);
-									$count=0;
-									foreach $tmatch (@matches) {
-										if ($REGEX == 1) {
-											$regex=$tmatch;
-											$regex =~ s/[^\\]\//\\\//g;
-										} else {
-											#$regex=wildcard($tmatch);
-											my $re = quotemeta $tmatch;
-											$re =~ s/\\\*/(.*)/g;
-											$regex = "^" . $re . "\$";
-										}
-										if($DEBUG>1) { LogMsg("Match: $regex against " . $strings[$count]); }
-										if($strings[$count] =~ /$regex/) {
-											$result=!$negate;
-										} else {
-											$result=$negate;
-										}
-										if(!$result) {
-											if($DEBUG > 1) { LogMsg("Match: No Match! (negate is $negate)"); }
-											break;
-										}
-										$count++;
-									}
-									#$result=Match($element,$match,$negate);
-								}
-								#if($result && !$negate || !$result && $negate) {}
-								if($result) {
-									# Yay - one of the alternatives matched
-									if($DEBUG > 1) { LogMsg("BREAKING out (result is $result, negate is $negate)"); }
-									last;
-								}
-							}
-							if($result) {
-								$ObjectiveMatch=1;
+
+	# Lets construct a token/data array in the order we wish to send things out, with
+	# appropriate data extrapolated.
+	my %tokens=();
+	my @tokenlist=();
+
+	$syscallname=$syscall{$event{$eventnum}{"syscall"}};
+	if(!$syscallname) {
+		# Fall back to the number.
+		$syscallname=$event{$eventnum}{"syscall"};
+	}
+	delete $event{$eventnum}{"syscall"};
+	$tokens{"event"}="$syscallname," . $event{$eventnum}{"datetime"};
+	push(@tokenlist,"event");
+	delete $event{$eventnum}{"datetime"};
+	if ($event{$eventnum}{"uid"} == -1) {
+		$uidname=$event{$eventnum}{"acct"};
+	} else {
+		$uidname=getuid($event{$eventnum}{"uid"});
+	}
+	$tokens{"uid"}=$event{$eventnum}{"uid"} . ($uidname?",$uidname":"");
+	push(@tokenlist,"uid");
+	delete $event{$eventnum}{"uid"};
+	$gidname=getgid($event{$eventnum}{"gid"});
+	$tokens{"gid"}=$event{$eventnum}{"gid"} . ($uidname?",$gidname":"");
+	push(@tokenlist,"gid");
+	delete $event{$eventnum}{"gid"};
+	$euidname=getuid($event{$eventnum}{"euid"});
+	$tokens{"euid"}=$event{$eventnum}{"euid"} . ($uidname?",$euidname":"");
+	push(@tokenlist,"euid");
+	delete $event{$eventnum}{"euid"};
+	$egidname=getgid($event{$eventnum}{"egid"});
+	$tokens{"egid"}=$event{$eventnum}{"egid"} . ($uidname?",$egidname":"");
+	push(@tokenlist,"egid");
+	delete $event{$eventnum}{"egid"};
+	$tokens{"process"}=$event{$eventnum}{"pid"} . "," . $event{$eventnum}{"comm"};
+	push(@tokenlist,"process");
+	delete $event{$eventnum}{"pid"};
+	delete $event{$eventnum}{"comm"};
+	$tokens{"return"}=$event{$eventnum}{"exit"} . "," . $event{$eventnum}{"success"};
+	push(@tokenlist,"return");
+	delete $event{$eventnum}{"exit"};
+	delete $event{$eventnum}{"success"};
+
+
+	foreach $key (sort keys %{$event{$eventnum}} ) {
+		# Resolve names
+		if($key =~ /^[eosa]uid(:[0-9]+)*$/ || $key =~ /^fsuid(:[0-9]+)*$/) {
+			$name=getuid($event{$eventnum}{$key});
+			if($name) {
+				$eventstring = $event{$eventnum}{$key} . ",$name";
+			}
+		} elsif($key =~ /^[eosa]gid(:[0-9]+)*$/ || $key =~ /^fsgid(:[0-9]+)*$/) {
+			$name=getgid($event{$eventnum}{$key});
+			if($name) {
+				$eventstring = $event{$eventnum}{$key} . ",$name";
+			}
+		} elsif($key =~ /^name(:[0-9]+)*$/ || $key =~ /^exe(:[0-9]+)*$/) {
+			$tpath=$event{$eventnum}{$key};
+			if($tpath =~ /^\// || !$event{$eventnum}{"cwd"}) {
+				$path = resolve_path($tpath);
+			} else {
+				$path = resolve_path($event{$eventnum}{"cwd"} . "/" . $tpath);
+			}
+			$eventstring = $path;
+		} else {
+			$eventstring = $event{$eventnum}{$key};
+		}
+		$tokens{$key}=$eventstring;
+		push(@tokenlist,$key);
+	}
+
+	# Ok, we have a range of token/value pairs to use for event checks.
+	delete $event{$eventnum};
+
+	# Format of the filters:
+	#$Filters{$ObjectiveCount}{$key}{$keymatch}{$ElementCount}{$AlternativeCount}=$alternative;
+	#$Filters{0}{event}{0}{0}{0}="execve";
+	$EventMatched=0;
+	$EventCrit=0;
+	foreach $objectivecount (keys(%Filters)) {
+		$ObjectiveMatch=0;
+		if($DEBUG>3) { LogMsg("LOOP: objcount $objectivecount\n"); }
+		foreach $key (keys(%{$Filters{$objectivecount}})) {
+			if($DEBUG>3) { LogMsg("LOOP: key $key\n"); }
+			if($tokens{$key}) {
+				foreach $keymatch (keys(%{$Filters{$objectivecount}{$key}})) {
+					if($DEBUG>3) { LogMsg("LOOP: keymatch $keymatch\n"); }
+					foreach $elementcount (keys(%{$Filters{$objectivecount}{$key}{$keymatch}})) {
+						if($DEBUG>3) { LogMsg("LOOP: elementcount $elementcount\n"); }
+						@elements=split(/,/,$tokens{$key});
+						if($DEBUG>3) { LogMsg("Key: $key - ELEMENTS: @elements\n"); }
+						$result=0;
+						foreach $alternativecount (keys(%{$Filters{$objectivecount}{$key}{$keymatch}{$elementcount}})) {
+							if($DEBUG>3) { LogMsg("LOOP: altcount $alternativecount\n"); }
+							$match=$Filters{$objectivecount}{$key}{$keymatch}{$elementcount}{$alternativecount};
+							$element=$elements[$elementcount];
+							$negate=$FilterTypes{$objectivecount}{$key}{$keymatch};
+							if($DEBUG>2) { LogMsg("Match element $element against $match (negate is $negate)\n"); }
+
+							# Special case:
+							if($match eq "*") {
+								$result=1;
 							} else {
-								$ObjectiveMatch=0;
+								@strings=split(/,/,$element);
+								@matches=split(/,/,$match);
+								$count=0;
+								foreach $tmatch (@matches) {
+									if ($REGEX == 1) {
+										$regex=$tmatch;
+										$regex =~ s/[^\\]\//\\\//g;
+									} else {
+										#$regex=wildcard($tmatch);
+										my $re = quotemeta $tmatch;
+										$re =~ s/\\\*/(.*)/g;
+										$regex = "^" . $re . "\$";
+									}
+									if($DEBUG>1) { LogMsg("Match: $regex against " . $strings[$count]); }
+									if($strings[$count] =~ /$regex/) {
+										$result=!$negate;
+									} else {
+										$result=$negate;
+									}
+									if(!$result) {
+										if($DEBUG > 1) { LogMsg("Match: No Match! (negate is $negate)"); }
+										break;
+									}
+									$count++;
+								}
+								#$result=Match($element,$match,$negate);
+							}
+							#if($result && !$negate || !$result && $negate) {}
+							if($result) {
+								# Yay - one of the alternatives matched
+								if($DEBUG > 1) { LogMsg("BREAKING out (result is $result, negate is $negate)"); }
 								last;
 							}
 						}
-						if(!$ObjectiveMatch) {
+						if($result) {
+							$ObjectiveMatch=1;
+						} else {
+							$ObjectiveMatch=0;
 							last;
 						}
 					}
-				} else {
-					# No token in this event that matches? Drop it.
-					$ObjectiveMatch=0;
-				}
-
-				if(!$ObjectiveMatch) {
-					last;
-				}
-			}
-			if($ObjectiveMatch) {
-				$EventMatched=1;
-				if($USECRITICALITY) {
-					$tcrit=$Criticality[$objectivecount];
-					if($tcrit > $EventCrit) {
-						$EventCrit=$tcrit;
+					if(!$ObjectiveMatch) {
+						last;
 					}
-				} else {
-					$EventCrit=$Criticality[$objectivecount];
-					last;
 				}
+			} else {
+				# No token in this event that matches? Drop it.
+				$ObjectiveMatch=0;
+			}
+
+			if(!$ObjectiveMatch) {
+				last;
 			}
 		}
-
-		if(!$EventMatched) {
-			# Ok, go to the next objective.
-			# We're not interested in this one.
-			next;
+		if($ObjectiveMatch) {
+			$EventMatched=1;
+			if($USECRITICALITY) {
+				$tcrit=$Criticality[$objectivecount];
+				if($tcrit > $EventCrit) {
+					$EventCrit=$tcrit;
+				}
+			} else {
+				$EventCrit=$Criticality[$objectivecount];
+				last;
+			}
 		}
-
-		# Craft our tokens into a single string.
-		$sendstring="criticality,$EventCrit";
-
-		foreach $key (@tokenlist) {
-			if($sendstring) { $sendstring .= "	"; }
-			$sendstring .= $key ."," . $tokens{$key};
-		}
-
-		# Send this out.
-		if($DEBUG) { LogMsg("OUTPUT: " . $sendstring); }
-
-		SendEvent("$FQDN	LinuxKAudit","$sendstring\n");
 	}
+
+	if(!$EventMatched) {
+		# Ok, go to the next objective.
+		# We're not interested in this one.
+		next;
+	}
+
+	# Craft our tokens into a single string.
+	$sendstring="criticality,$EventCrit";
+
+	foreach $key (@tokenlist) {
+		if($sendstring) { $sendstring .= "	"; }
+		$sendstring .= $key ."," . $tokens{$key};
+	}
+
+	# Send this out.
+	if($DEBUG) { LogMsg("OUTPUT: " . $sendstring); }
+
+	SendEvent("$FQDN	LinuxKAudit","$sendstring\n");
 }
 
 sub SendEvent() {
@@ -667,16 +705,33 @@ sub LoadConfig() {
 								if($key eq "event") {
 									# put the events in a temporary list
 									$temp_EventsON{$alternative}=$SUCCESS + $FAILURE;
+#									if ($alternative eq "open") {
+#										# put OPEN objectives at the start to try and reduce the load
+#										#DMM
+#										$NewFilters{0} = $Filters{$TEMPObjectiveCount};
+#										$NewCriticality[0]=$Criticality[$TEMPObjectiveCount];
+#										delete $Filters{$TEMPObjectiveCount};
+#										$TEMPObjectiveCount = 0;
+#										foreach $objcount (keys(%Filters)) {
+#											$NewFilters{$objcount+1} = $Filters{$objcount};
+#											$NewCriticality[$objcount+1] = $Criticality[$objcount];
+#										}
+#										$Filters = $NewFilters;
+#										$Criticality = $NewCriticality;
+#										$Filters{$TEMPObjectiveCount}{$key}{$keymatch}{$ElementCount}{$AlternativeCount}=$alternative;
+#										%NewFilters=();
+#										%NewCriticality=();
+#									}
 								} elsif($key eq "return") {
 									# if we are only looking for a specific return code,
 									# only enable the necessary auditing
 									if ($alternative eq "no") {
 										foreach $eventid (keys(%temp_EventsON)) {
-											$temp_EventsON{$eventid}=$FAILURE;
+											$temp_EventsON{$eventid}= $temp_EventsON{$eventid} | $FAILURE;
 										}
 									} elsif ($alternative eq "yes") {
 										foreach $eventid (keys(%temp_EventsON)) {
-											$temp_EventsON{$eventid}=$SUCCESS;
+											$temp_EventsON{$eventid}= $temp_EventsON{$eventid} | $SUCCESS;
 										}
 									}
 								}
@@ -694,6 +749,11 @@ sub LoadConfig() {
 					$EventsON{$eventid} = $temp_EventsON{$eventid};
 					delete $temp_EventsON{$eventid};
 				}
+#				#DMM
+#				foreach $alternativecount (keys(%{$Filters{$TEMPObjectiveCount}{"event"}{0}{0}})) {
+#					$match=$Filters{$TEMPObjectiveCount}{"event"}{0}{0}{$alternativecount};
+#					LogMsg("EVENTMATCH: altcount $alternativecount $match\n");
+#				}
 
 				$ObjectiveCount++;
 			}
@@ -793,7 +853,7 @@ sub Match {
 
 sub OpenOutput() {
 	my $line = shift;
-	($type,$destination,$opt)=split(/[=:]/,$line);
+	($type,$destination,$opt,$opt2)=split(/[=:]/,$line);
 	if($type eq "file") {
 		if($destination eq "stdout") {
 			open($fp,">-");
@@ -808,13 +868,21 @@ sub OpenOutput() {
 		return(0);
 	} elsif($type eq "network") {
 		# Destination will be an ip or dns name
-		# opt will be the port.
+		# opt will be the port, opt2 the protocol
 		$Port=6161;
 		if($opt > 0 && $opt < 65536) {
 			$Port=$opt;
 		}
 
-		socket($tsocket, PF_INET, SOCK_DGRAM, getprotobyname("udp")) or return(0);
+#ifdef SUPPORTED_SNARE
+		if ($opt2 eq "tcp") {
+			socket($tsocket, PF_INET, SOCK_STREAM, getprotobyname("tcp")) or return(0);
+		} else {
+			socket($tsocket, PF_INET, SOCK_DGRAM, getprotobyname("udp")) or return(0);
+		}
+#else
+		#socket($tsocket, PF_INET, SOCK_DGRAM, getprotobyname("udp")) or return(0);
+#endif //SUPPORTED_SNARE
 		if($destination =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) {
 			# IP Address
 			$ipaddr = inet_aton($destination);
@@ -832,6 +900,7 @@ sub OpenOutput() {
 		$OUTPUTNET{$OUTPUTNET{"SocketCount"}}{"Socket"}=$tsocket;
 		$OUTPUTNET{$OUTPUTNET{"SocketCount"}}{"PortAddr"}=$portaddr;
 		$OUTPUTNET{$OUTPUTNET{"SocketCount"}}{"Port"}=$Port;
+		$OUTPUTNET{$OUTPUTNET{"SocketCount"}}{"Line"}=$line;
 		$OUTPUTNET{"SocketCount"}++;
 	}
 	return(0);
@@ -875,6 +944,24 @@ sub resolve_path {
 }
 
 
+sub getuname {
+	my($name)=@_;
+	if($unamecache{"$name"} eq "-") {
+		return(-1);
+	} elsif($unamecache{"$name"}) {
+		return($unamecache{"$name"});
+	} else {
+		$id=getpwnam($name);
+		if($id) {
+			$unamecache{"$name"}=$id;
+			return($id);
+		} else {
+			$unamecache{"$name"}="-";
+			return(-1);
+		}
+	}
+}
+
 sub getuid {
 	my($id)=@_;
 	if($uidcache{"$id"} eq "-") {
@@ -927,7 +1014,6 @@ sub Restart {
 	#}
 
 	# Flush our buffers
-	SendEvents($FLUSHNOW);
 	CloseOutputs();
 	sleep(1);
 
@@ -936,8 +1022,6 @@ sub Restart {
 	$eventsent=0;
 	$lasteventnum=0;
 
-	$timecheck=time();
-
 	LoadConfig();
 	StartWeb();
 	# Set up our audit configuration based on our objectives.
