@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
 # Snare Audit Dispatcher for Linux - Web Server
-# (c) Copyright 2006 InterSect Alliance Pty Ltd
+# (c) Copyright 2007 InterSect Alliance Pty Ltd
 #
 
 use IO::Socket;
@@ -31,7 +31,6 @@ sub WebServerSetup() {
 	$ppid=getppid();
 
 	$WEBCONTINUE=1;
-	$DNSTIMEOUT=120;
 
 	$SIG{QUIT}='WebTerminate';
 
@@ -41,6 +40,8 @@ sub WebServerSetup() {
 	$ATTRIBUTE_EVENTS="chmod,chown,lchown,fchmod,fchown,fchown32,lchown32,chown32";
 	$ADMIN_EVENTS="mount,umount,umount2,settimeofday,swapon,swapoff,reboot,setdomainname,create_module,delete_module,quotactl";
 	$NETWORK_EVENTS="socketcall";
+	$AUTH_EVENTS="login_auth,login_start,logout";
+	$ACCT_EVENTS="acct_mgmt";
 	
 	
 	$server = IO::Socket::INET->new(Proto=>'tcp',
@@ -380,6 +381,162 @@ sub HandleWebPages() {
 		print $theclient "<div align=center><input type=submit value=\"Change Configuration\"></div>\n";
 		print $theclient "</form>\n";
 
+		SnareTail($theclient);
+		Tail($theclient);
+	} elsif($url =~ /^\/objectives$/) {
+		ReadConfig();
+		HTMLHeader($theclient,"html");
+		Head($theclient,"Snare Agent for Linux");
+		SnareHead($theclient);
+		print $theclient "<center><h1>Objective Control</h1></center>";
+		# Maybe some javascript here to 'suggest' appropriate
+		# settings based on the users match token?
+		# eg: if process, then match = *,ls
+		print $theclient "<form action=setobjective>\n";
+
+		print $theclient "<center><table style=\"border: 1px solid black;\">";
+		print $theclient "<tr style=\"background: #ffffcc;\"><td><b>Criticality</b></td><td><b>Events</b></td><td><b>Return</b></td><td><b>User</b></td><td><b>Match</b></td></tr>\n";
+		$toggle=0;
+		$count=0;
+		@arraycheck=keys(%{$Config{"OBJECTIVES"}});
+		$arraysize=@arraycheck;
+
+		foreach $element (sort(keys(%{$Config{"OBJECTIVES"}}))) {
+			@elements=split(/[ \t]*([a-z0-9:]+)(\!*=)([^ \t]+)[ \t]*/,$Config{"OBJECTIVES"}{$element});
+			$type=0;
+			%ObjectiveMatch=();
+			foreach $component (@elements) {
+				if($component =~ /^[ \t]*$/ || $component eq "") {
+					next;
+				}
+				if($type==0) {
+					$token=$component;
+					$type++;
+				} elsif($type == 1) {
+					$comp=$component;
+					$type++;
+				} elsif($type == 2) {
+					$match=$component;
+					if ($token =~ /(criticality|uid|return|event)/) {
+						$ObjectiveMatch{$token}=$match;
+						$ObjectiveCompare{$token}=$comp;
+					} else {
+						$elements=keys(%{$ObjectiveMatch{$token}});
+						$ObjectiveMatch{$token}{$elements}=$match;
+						$ObjectiveCompare{$token}{$elements}=$comp;
+					}
+					$type=0;
+				}
+			}
+			# Ok, we now have a match term, and comparison (=/!=) for each token.
+
+			$crit=$ObjectiveMatch{"criticality"};
+			delete $ObjectiveMatch{"criticality"};
+			delete $ObjectiveCompare{"criticality"};
+
+			$events=$ObjectiveMatch{"event"};
+			$events =~ s/[\(\)]+//g;
+			delete $ObjectiveMatch{"event"};
+			delete $ObjectiveCompare{"event"};
+
+			if($count) {
+				print $theclient "<tr style=\"height: 5px;\"><td colspan=5></td></tr>\n";
+			}
+			if($toggle) {
+				$toggle=0;
+				print $theclient "<tr style=\"background: #ffffcc\" valign=top>";
+			} else {
+				$toggle=1;
+				print $theclient "<tr style=\"background: #ffffaa\" valign=top>";
+			}
+
+			print $theclient "<td>";
+			if ($crit lt 1 || $crit gt 4) {
+				print $theclient "0 - clear";
+			} elsif ($crit eq 1) {
+				print $theclient "1 - green";
+			} elsif ($crit eq 2) {
+				print $theclient "2 - yellow";
+			} elsif ($crit eq 3) {
+				print $theclient "3 - orange";
+			} elsif ($crit eq 4) {
+				print $theclient "4 - red";
+			}
+			print $theclient "</td>";
+			print $theclient "<td>";
+			if($events eq $OPEN_EVENTS) {
+				print $theclient "OPEN_EVENTS";
+			} elsif($events eq $REMOVE_EVENTS) {
+				print $theclient "REMOVE_EVENTS";
+			} elsif($events eq $ATTRIBUTE_EVENTS) {
+				print $theclient "ATTRIBUTE_EVENTS";
+			} elsif($events eq $PROCESS_EVENTS) {
+				print $theclient "PROCESS_EVENTS";
+			} elsif($events eq $ADMIN_EVENTS) {
+				print $theclient "ADMIN_EVENTS";
+			} elsif($events eq $NETWORK_EVENTS) {
+				print $theclient "NETWORK_EVENTS";
+			} elsif($events eq $AUTH_EVENTS) {
+				print $theclient "AUTH_EVENTS";
+			} elsif($events eq $ACCT_EVENTS) {
+				print $theclient "ACCT_EVENTS";
+			} else {
+				print $theclient "$events";
+			}
+			print $theclient "</td>";
+
+
+			print $theclient "<td>";
+			if ($ObjectiveMatch{"return"} !~ /^\*,(yes|no)$/) {
+				print $theclient "ANY";
+			} elsif ($ObjectiveMatch{"return"} !~ /^\*,yes$/) {
+				print $theclient "SUCCESS";
+			} elsif ($ObjectiveMatch{"return"} !~ /^\*,no$/) {
+				print $theclient "FAILURE";
+			}
+			print $theclient "</td>";
+
+			print $theclient "<td>";
+			delete $ObjectiveMatch{"return"};
+			delete $ObjectiveCompare{"return"};
+
+
+			$user=$ObjectiveMatch{"uid"};
+			$userincexc=$ObjectiveCompare{"uid"};
+			$user=~s/[\(\)]+//g;
+			$user =~ s/^\*,//;
+			delete $ObjectiveMatch{"uid"};
+			delete $ObjectiveCompare{"uid"};
+
+			if ($userincexc !~ /^!*=$/) {
+				print $theclient "ANY";
+			} elsif ($userincexc eq "=") {
+				print $theclient "INCLUDE:$user";
+			} elsif ($userincexc eq "!=") {
+				print $theclient "EXCLUDE:$user";
+			}
+			print $theclient "</td>";
+			print $theclient "<td>";
+
+			# For each of the match terms specified by the user.
+			# Token/value
+			foreach $matchtype (keys(%ObjectiveMatch)) {
+				$token=$matchtype;
+				foreach $elem (keys(%{$ObjectiveMatch{$token}})) {
+					$value=$ObjectiveMatch{$matchtype}{$elem};
+					$cmp=$ObjectiveCompare{$matchtype}{$elem};
+					print $theclient "$token $cmp $value<br>\n";
+				}
+			}
+			print $theclient "<br>";
+
+			print $theclient "</td>";
+			print $theclient "</tr>\n";
+			$count++;
+		}
+		print $theclient "</table></center>\n";
+		print $theclient "</form>";
+
 		SnareTail($theclient);
 		Tail($theclient);
 	} elsif($url =~ /^\/(set)*objective/) {
@@ -432,6 +589,10 @@ sub HandleWebPages() {
 						$events=$ADMIN_EVENTS;
 					} elsif($eventselect =~ /^NETWORK_EVENTS$/) {
 						$events=$NETWORK_EVENTS;
+					} elsif($eventselect =~ /^AUTH_EVENTS$/) {
+						$events=$AUTH_EVENTS;
+					} elsif($eventselect =~ /^ACCT_EVENTS$/) {
+						$events=$ACCT_EVENTS;
 					} elsif($eventselect =~ /^OTHER_EVENTS$/) {
 						# Does the events information look to be valid?
 						if($events !~ /^[a-z0-9,_]+$/) {
@@ -497,6 +658,8 @@ sub HandleWebPages() {
 			$Config{"OBJECTIVES"}{1}="	criticality=3	event=$PROCESS_EVENTS	uid=*,root";
 			$Config{"OBJECTIVES"}{2}="	criticality=3	event=($ADMIN_EVENTS)	uid=*,root";
 			$Config{"OBJECTIVES"}{3}="	criticality=0	event=$PROCESS_EVENTS	exe=*passwd*";
+			$Config{"OBJECTIVES"}{4}="	criticality=0	event=$AUTH_EVENTS";
+			$Config{"OBJECTIVES"}{5}="	criticality=0	event=$ACCT_EVENTS";
 			$rc=WriteConfig();
 		} elsif($url =~ /\?SOX=1$/) {
 			delete($Config{"OBJECTIVES"});
@@ -505,6 +668,8 @@ sub HandleWebPages() {
 			$Config{"OBJECTIVES"}{2}="	criticality=3	event=($ADMIN_EVENTS)	uid=*,root";
 			$Config{"OBJECTIVES"}{3}="	criticality=0	event=$PROCESS_EVENTS	exe=*passwd*";
 			$Config{"OBJECTIVES"}{4}="	criticality=0	event=$NETWORK_EVENTS";
+			$Config{"OBJECTIVES"}{5}="	criticality=0	event=$AUTH_EVENTS";
+			$Config{"OBJECTIVES"}{6}="	criticality=0	event=$ACCT_EVENTS";
 			$rc=WriteConfig();
 		} elsif($url  =~ /\?PCI=1$/) {
 			delete($Config{"OBJECTIVES"});
@@ -513,6 +678,8 @@ sub HandleWebPages() {
 			$Config{"OBJECTIVES"}{2}="	criticality=3	event=($ADMIN_EVENTS)	uid=*,root";
 			$Config{"OBJECTIVES"}{3}="	criticality=0	event=$PROCESS_EVENTS	exe=*passwd*";
 			$Config{"OBJECTIVES"}{4}="	criticality=0	event=$NETWORK_EVENTS";
+			$Config{"OBJECTIVES"}{5}="	criticality=0	event=$AUTH_EVENTS";
+			$Config{"OBJECTIVES"}{6}="	criticality=0	event=$ACCT_EVENTS";
 			$rc=WriteConfig();
 		}
 		print $theclient "<form action=setobjective>\n";
@@ -603,6 +770,11 @@ sub HandleWebPages() {
 			if($events eq $NETWORK_EVENTS) { $selected=6; }
 			print $theclient "<option value=\"NETWORK_EVENTS\"" . ($selected==6?" selected ":"") . " onclick='document.getElementById(\"ObjEvents-$count\").value=\"$NETWORK_EVENTS\";'>Network Socketcall Events</option>";
 
+			if($events eq $AUTH_EVENTS) { $selected=7; }
+			print $theclient "<option value=\"AUTH_EVENTS\"" . ($selected==7?" selected ":"") . " onclick='document.getElementById(\"ObjEvents-$count\").value=\"$AUTH_EVENTS\";'>Authentication Events</option>";
+			if($events eq $ACCT_EVENTS) { $selected=8; }
+			print $theclient "<option value=\"ACCT_EVENTS\"" . ($selected==8?" selected ":"") . " onclick='document.getElementById(\"ObjEvents-$count\").value=\"$ACCT_EVENTS\";'>Account Administration Events</option>";
+
 			if(!$selected) {
 				print $theclient "<option value=\"OTHER_EVENTS\" selected>Other Events (described below)</option>";
 			} else {
@@ -1218,7 +1390,7 @@ sub Tail() {
 		"    </tbody>\n" .
 		"   </table>\n" .
 		"<center>\n" .
-		"<br><br><font size=-1>(c) <a href=\"http://www.intersectalliance.com\">Intersect Alliance</a> Pty Ltd 1999-2006. " .
+		"<br><br><font size=-1>(c) <a href=\"http://www.intersectalliance.com\">Intersect Alliance</a> Pty Ltd 1999-2007. " .
 		"This site is powered by <a href=\"http://www.intersectalliance.com/projects/index.html\">SNARE for Linux.</a></font>" .
 		"</center>\n" .
 		"</td></tr></tbody></table>\n" .
